# -*- coding: utf-8 -*-
#    Copyright (C) 2007 Stewart Adam
#    This file is part of audio-convert-mod.

#    audio-convert-mod is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    audio-convert-mod is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with audio-convert-mod; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
audio-convert-mod package initialization
"""

__author__ = "Stewart Adam <s.adam@diffingo.com>"
__status__ = "beta"
__version__ = "3.45.2"

try:
  import gtk
except:
  print 'Critical error: cannot import GTK.'
  print 'Please make sure an X display is running and the DISPLAY variable is set.'

import sys
import traceback
import exceptions
import time
from threading import Thread, Event
import webbrowser
import time
  

global isVerbose
isVerbose = False
def setVerbose(bool):
  global isVerbose
  isVerbose = bool

def verbose(string):
  if isVerbose:
    print 'VERBOSE: %s' % string

class FuncAsThread(Thread):
    """ Run a function as a new thread. """
    def __init__(self, functorun):
        Thread.__init__(self)
        self.functorun = functorun
        
    def run(self):
        try:
            retval = self.functorun()
            return retval
        except exceptions.SystemExit:
            sys.exit(0)
        except: # catch other exception and write it to the logger.
            etype = sys.exc_info()[0]
            evalue = sys.exc_info()[1]
            etb = traceback.extract_tb(sys.exc_info()[2])
            print 'An error occurred while running a subprocess!'
            print 'Traceback: \n'
            for tub in etb:
                f, l, m, c = tub # file,lineno, function, codeline
                print 'File: %(a)s, line %(b)s, in %(c)s' % {'a': f, 'b': l, 'c': m}
                print '    %s ' % c
                print '%s: %s' % (etype, evalue)
            self.exception = etype(evalue)
            sys.exit(1)

def runFuncAsThread(functorun):
    """ Runs a function as a new thread, updating gtk. """
    thread = FuncAsThread(functorun)
    thread.start()
    return thread

def doGtkEvents():
    """ Process gtk events """
    while gtk.events_pending():
        gtk.main_iteration()

def busyCursor(rootWindow, insensitive=False):
    """ Set busy cursor in rootWindow and make it insensitive if selected """
    rootWindow.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
    if insensitive:
        rootWindow.set_sensitive(False)
    doGtkEvents()

def normalCursor(rootWindow):
    """ Set Normal cursor in rootWindow and make it sensitive """
    rootWindow.window.set_cursor(None)
    rootWindow.set_sensitive(True)        
    doGtkEvents()

def liststoreIntoArray(liststore):
  """ Turn a liststore into an array filled with it's rows and columns """
  def callback(model, path, iter, user_data):
    """ Just returns the contents of the row as a list """
    noColumns, allRows = user_data[0], user_data[1] 
    currColumn = 0
    theRow = []
    # Append each value/column into the row list
    while currColumn < noColumns:
      theRow.append(model.get_value(iter, currColumn))
      currColumn = currColumn + 1
    # add this row to allRows
    allRows.append(theRow)
  # get number of columns, used in the loop in callback()
  noColumns = liststore.get_n_columns()
  allRows = []
  # run for each row - All rows get added to allRows
  liststore.foreach(callback, [noColumns, allRows])
  return allRows

def arrayIndex(array, text):
  """ A list's 'index' function for a arrays (multi-level lists)  """
  allMatches = []
  for i in array:
    try:
      # if found, let's get the key and index
      index = i.index(text)
      key = array.index(i)
      # if it doesn't exist, the above fails and we skip to except
      allMatches.append([key, index])
    except ValueError:
      # if not, just my mouth shut and keep executing
      pass
  return allMatches
  raise ValueError('array.index(text): text not in array')

def liststoreIndex(liststore, text):
  """ A list's 'index' function for a liststore  """
  # get the list
  liststoreArray = liststoreIntoArray(liststore)
  return liststoreArray, arrayIndex(liststoreArray, text)

def set_text_markup(label, text):
  """ set a gtk.Label's text with markup """
  label.set_text(text)
  label.set_use_markup(True)


