# -*- coding: utf-8 -*-
#  Copyright (C) 2007 Stewart Adam
#  This file is part of audio-convert-mod.

#  audio-convert-mod is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  audio-convert-mod is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with audio-convert-mod; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
The callbacks for everything Glade.
"""
import gtk
import gtk.glade
import gobject
import webbrowser
import rat.hig
import time
import shutil
import ConfigParser
import re

import audio_convert_mod
from audio_convert_mod import verbose
import widgets, formats
from i18n import _
from const import *
from const import _readPrefs, _writePrefs

def escape(uri):
    "Convert each space to %20, etc"
    _to_utf8 = codecs.getencoder('utf-8')
    return re.sub('[^:-_./a-zA-Z0-9]',
        lambda match: '%%%02x' % ord(match.group(0)),
        _to_utf8(uri)[0])

def unescape(uri):
    "Convert each %20 to a space, etc"
    if '%' not in uri: return uri
    return re.sub('%[0-9a-fA-F][0-9a-fA-F]',
        lambda match: chr(int(match.group(0)[1:], 16)),
        uri)

def get_local_path(uri):
    """Convert 'uri' to a local path and return, if possible. If 'uri'
    is a resource on a remote machine, return None. URI is in the escaped form
    (%20 for space)."""
    if not uri:
        return None

    if uri[0] == '/':
        if uri[1:2] != '/':
            return unescape(uri)    # A normal Unix pathname
        i = uri.find('/', 2)
        if i == -1:
            return None    # //something
        if i == 2:
            return unescape(uri[2:])    # ///path
        remote_host = uri[2:i]
        if remote_host == our_host_name():
            return unescape(uri[i:])    # //localhost/path
        # //otherhost/path
    elif uri[:5].lower() == 'file:':
        if uri[5:6] == '/':
            return get_local_path(uri[5:])
    elif uri[:2] == './' or uri[:3] == '../':
        return unescape(uri)
    return None

class UI(gtk.glade.XML):
  """Base class for UIs loaded from glade."""
  def __init__(self, filename, rootname,domain=None):
    """
      Initialize a new instance.
      `filename' is the name of the .glade file containing the UI hierarchy.
      `rootname' is the name of the topmost widget to be loaded.
      `gladeDir' is the name of the directory, relative to the Python
      path, in which to search for `filename'
    """
    if domain:
      gtk.glade.XML.__init__(self, filename, root=None, domain=domain)
    else:
      gtk.glade.XML.__init__(self, filename, root=None, domain=None)
    self.filename = filename
    self.root = self.get_widget(rootname)

  def __getattr__(self, name):
    """Look up an as-yet undefined attribute, assuming it's a widgets."""
    result = self.get_widget(name)
    if result is None:
      raise AttributeError("Can't find widget %s in %s.\n" %
                 (`name`, `self.filename`))
    
    # Cache the widget to speed up future lookups.  If multiple
    # widgets in a hierarchy have the same name, the lookup
    # behavior is non-deterministic just as for libglade.
    setattr(self, name, result)
    return result

class Controller:
  """ Base class for all controllers of glade-derived UIs. """

  def _getAllMethods(self):
    """ Get a dictionary of all methods in self's class hierarchy. """
    result = {}

    # Find all callable instance/class attributes.  This will miss
    # attributes which are "interpreted" via __getattr__.  By
    # convention such attributes should be listed in
    # self.__methods__.
    allAttrNames = self.__dict__.keys() + self._getAllClassAttributes()
    for name in allAttrNames:
      value = getattr(self, name)
      if callable(value):
        result[name] = value
    return result

  def _getAllClassAttributes(self):
    """ Get a list of all attribute names in self's class hierarchy. """
    nameSet = {}
    for currClass in self._getAllClasses():
      nameSet.update(currClass.__dict__)
    result = nameSet.keys()
    return result

  def _getAllClasses(self):
    """ Get all classes in self's heritage. """
    result = [self.__class__]
    i = 0
    while i < len(result):
      currClass = result[i]
      result.extend(list(currClass.__bases__))
      i = i + 1
    return result

  def drag_data_received(self, w, context, x, y, data, info, time):
      paths = []
      if data and data.format == 8:
            for i in data.data.split('\r\n'):
                if i != "" or i != None:
                    path = get_local_path(i)
                    if path != None:
                        paths.append(path)
      widgets.addToSelectFilesFilesTreeview(self, paths)
      context.finish(True, False, time)



  def __init__(self, gladeFile, paths):
    """ Initialize a new instance.
      `gladeFile' is the location of a glade file.
      `paths' is a list of paths to add on startup (if any) """
    def setIfNonExistant(config, option, value):
      """ Sets the value of option to value if not found already in config """
      options = config.options('Preferences')
      if options.count(option) == 0:
        prefs.set('Preferences', option, value)
    # Create and ui object contains the widgets
    self.ui = UI(gladeFile, 'main', 'audio-convert-mod')
    self.ui.signal_autoconnect(self._getAllMethods())
    # Statusbar
    #self.statusbar = widgets.StatusBar(self.ui.statusbar1)
    # Transient Windows
    self.ui.about.set_transient_for(self.ui.main)
    self.ui.chooser.set_transient_for(self.ui.main)
    self.ui.features.set_transient_for(self.ui.main)
    self.ui.errorDetails.set_transient_for(self.ui.main)
    self.ui.tags.set_transient_for(self.ui.main)
    self.ui.prefs.set_transient_for(self.ui.main)

    liststore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
    self.ui.main2QualityCombobox.set_model(liststore)
    cell = gtk.CellRendererText()
    self.ui.main2QualityCombobox.clear()
    self.ui.main2QualityCombobox.pack_start(cell, True)
    self.ui.main2QualityCombobox.add_attribute(cell, 'text', 1)
    widgets.setupSelectFilesFilesTreeview(self)
    widgets.setupFeaturesTreeview(self)
    self.processId = None

    self.checkFormats()
    self._setDefaults()

    # setup config dir
    configDir = '%s/.audio-convert-mod' % USERHOME
    if not os.path.exists(configDir):
      try:
        os.mkdir(configDir)
      except OSError, error:
        rat.hig.error(_('Critical Error'),
                      _('Could not create the configuration directory `~/.audio-convert-mod\':\n%s' % error),
                      title=_('Critical Error')
                      )
        sys.exit(1)
    elif not os.path.isdir(configDir):
        rat.hig.error(_('Critical Error'),
                      _('The configuration directory is a file! Please remove/rename `~/.audio-convert-mod.\''),
                         title=_('Critical Error')
                      )
        sys.exit(1)
    elif not os.access(configDir, os.R_OK) and os.access(configDir, os.W_OK):
        rat.hig.error(_('Critical Error'),
                      _('You do not have access to the configuration directory ' + \
                        '`~/.audio-convert-mod\'.\nRead and write access is required.'),
                         title=_('Critical Error')
                      )
        sys.exit(1)
    # it exists, is a directory, and is writable. Fair enough.
    if not os.path.exists(PREFLOC):
      fh = open(PREFLOC, 'w')
      fh.write('')
      fh.close()
    prefs = _readPrefs()
    if not prefs.has_section('Preferences'):
      prefs.add_section('Preferences')
      defaults = [['ShowTrayIcon', 1], ['BlinkTrayIcon', 1], ['PauseOnErrors', 1]]
      for pair in defaults:
        setIfNonExistant(prefs, pair[0], pair[1])
    if not prefs.has_option('Preferences', 'Version'): # version was added in 3.45.1
      prefs.set('Preferences', 'Version', '3.45.0')
    _writePrefs(prefs)
    oldVersion = prefs.get('Preferences', 'Version')
    # only if it's a version mismatch should we import
    if oldVersion != VERSION:
      # This lets us do what needs to be done for version X and _above_
      fromHereUp = False
      # --------------------------------------------------------
      # oldest first, newest last in for the order of IF clauses
      # --------------------------------------------------------
      if oldVersion == '3.45.0' or fromHereUp == True:
        fromHereUp == True
        if not prefs.has_section('Defaults'):
          prefs.add_section('Defaults')
        prefs.set('Defaults', 'Format', 'WAV')
        prefs.set('Defaults', 'Quality', '0')
        prefs.set('Defaults', 'Extension', '0')
        prefs.set('Defaults', 'FileExists', '1') # append
        prefs.set('Defaults', 'Metadata', '1') # copy
        prefs.set('Defaults', 'successfulConversion', '1') # leave in place
        prefs.set('Defaults', 'outputFolder', USERHOME) # home


    # finally...
    prefs.set('Preferences', 'version', '3.45.1')
    _writePrefs(prefs)
    self._setupTrayIcon()
    if int(_readPrefs().get('Preferences', 'ShowTrayIcon')) == 1:
      self.trayicon.set_visible(True)
    else:
      self.trayicon.set_visible(False)

    # now add those paths
    if paths != []:
      fileList = []
      for path in paths:
        if os.path.exists(path):
          if os.path.isdir(path): # a dir
            files = os.listdir(path)
            files.sort()
            for filename in files:
              fileList.append('%s/%s' % (path, filename) )
          else: # a file
            fileList.append('%s' % path )
        else: # file didn't exist
          verbose('Skipping non-existant file `%s\'' % path)
      errors = widgets.addToSelectFilesFilesTreeview(self, fileList)
      if errors != []:
        self.ui.errorDetails.show()
        audio_convert_mod.set_text_markup(self.ui.errorDetailsTitleLabel, _('<span size="large" weight="bold">Error adding files</span>'))
        self.ui.errorDetails.set_title(_('Error adding files'))
        self.ui.errorDetailsDescriptionLabel.set_text(_('Not all files could be added.'))
        self.ui.errorDetailsTextview.get_buffer().set_text('\n'.join(errors))
    self.ui.main1FilesTreeview.connect('drag_data_received', self.drag_data_received)

  def _setDefaults(self):
    self.ui.mainControlNotebook.set_show_tabs(False)
    # Default to 0...
    self.ui.aboutNotebook.set_current_page(0)
    self.ui.mainControlNotebook.set_current_page(0)
    # Labels
    audio_convert_mod.set_text_markup(self.ui.aboutVersionLabel, ('<span size="xx-large" weight="bold">audio-convert-mod %s</span>' % VERSION))
    self.ui.main3CurrentFileLabel.set_label(_('None'))
    self.ui.main2OutputFolderFilechooser.set_current_folder(USERHOME)
    self.ui.main1FilesTreeview.get_model().clear()
    self.ui.featuresFeaturesTreeview.get_model().clear()
    self.ui.main2ExistsRadio1.set_active(True)
    self.ui.main2MetatagRadio1.set_active(True)
    self.ui.main2UponSuccessRadio1.set_active(True)
    self.ui.main2OutputFolderRadio1.set_active(True)

  def _setupTrayIcon(self):
    """ Sets up the tray icon """
    pix = self.ui.main.render_icon(gtk.STOCK_CONVERT, gtk.ICON_SIZE_MENU)
    #pix = gtk.gdk.pixbuf_new_from_file("/usr/share/pixmaps/fwbackups.svg")
    self.trayicon = gtk.status_icon_new_from_pixbuf(pix)
    self.trayicon.set_from_pixbuf(pix)
    #self.trayicon = gtk.status_icon_new_from_stock(gtk.STOCK_COPY)
    self.trayicon.connect("popup_menu", self._Popup)
    self.trayicon.connect("activate", self._clicked)
    self.setStatus(_('Idle'))

  def setStatus(self, status, blink=False):
    """ Sets the status icon and tray label """
    self.trayicon.set_tooltip(_('audio-convert-mod - %s' % status))
    if int(_readPrefs().get('Preferences', 'BlinkTrayIcon')):
      self.trayicon.set_blinking(blink)
    self.ui.main3StatusLabel.set_text(status)

  def _clicked(self, status):
    """ Tray icon is clicked.
      status: the gtk.StatusIcon
    """
    # use me for menu on left click
    """def menu_pos(menu):
      return gtk.status_icon_position_menu(menu, self.trayicon)
    self.ui.TrayMenu1.popup(None, None, menu_pos, 0,
                gtk.get_current_event_time())"""
    if self.trayicon.get_blinking():
      self.trayicon.set_blinking(False)
      self.ui.main.hide()
      self.ui.main.show()
    else:
      if self.ui.main.get_property('visible'):
        self.ui.main.hide()
        self.ui.show_conversion_window1.set_active(False)
      else:
        self.ui.main.show()
        self.ui.show_conversion_window1.set_active(True)

  def _Popup(self, status, button, time):
    """ Popup the menu at the right position """
    def menu_pos(menu):
      return gtk.status_icon_position_menu(menu, self.trayicon)
    
    self.ui.trayMenu.popup(None, None, menu_pos, button, time)
    self.ui.trayMenu.show()

  ###
  ### TRAY ICON ###
  ###

  def on_show_conversion_window1_activate(self, widget, event=None):
    """ Hide/show the main window """
    if self.ui.show_conversion_window1.get_active():
      self.ui.main.show()
    else:
      self.ui.main.hide()

  def on_show_tray_icon1_activate(self, widget, event=None):
    """ Hide/show the main window """
    active = self.ui.show_tray_icon1.get_active()
    self.ui.prefsShowTrayIconCheck.set_active(active)
    self.trayicon.set_visible(active)
  # --

  def on_preferences2_activate(self, widget, event=None):
    """ Show preferences """
    self.on_preferences1_activate(None)

  def on_show_features2_activate(self, widget, event=None):
    """ Show features """
    self.on_show_features1_activate(widget)

  # --
  def on_quit2_activate(self, widget):
    """ Quit """
    self.main_close(self)
    return False



  def checkFormats(self):
    formats.recheck()
    active = self.ui.main2FormatCombobox.get_active()
    model = self.ui.main2QualityCombobox.get_model().clear()
    model = self.ui.main2FormatCombobox.get_model()
    model.clear()
    for format in formats.FORMATS:
      if format.get()[1] == True:
        model.append([format.__class__.__name__.upper()])
    if active >= 0:
      self.ui.main2FormatCombobox.set_active(active)
    else:
      self.ui.main2FormatCombobox.set_active(0)

  def refreshFeatures(self):
    """ Refreshes the checkmarks in Features window """
    def convert(label, formatObj):
      """ Get an object, return the right liststore entry """
      decode, encode = gtk.STOCK_NO, gtk.STOCK_NO
      if formatObj.get()[0] == True:
        decode = gtk.STOCK_YES
      if formatObj.get()[1] == True:
        encode = gtk.STOCK_YES
      return [label, decode, encode]
    model = self.ui.featuresFeaturesTreeview.get_model()
    model.clear()
    model.append(convert('MP3 (lame)', formats.MP3))
    model.append(convert('OGG (oggenc|oggdec)', formats.OGG))
    model.append(convert('MPC (mpcdec|mpcenc)', formats.MPC))
    model.append(convert('APE (mac)', formats.APE))
    model.append(convert('AAC (faad|faac)', formats.AAC))
    model.append(convert('AC3 (a52dec|ffmpeg)', formats.AC3))
    model.append(convert('FLAC (flac)', formats.FLAC))
    model.append(convert('WV (wavpack|wvunpak)', formats.WV))


  ###
  ### WINDOW WRAPPERS ###
  ###

  def hide(self, widget, event=None):
    """ Wrapper for closing a window non-destructively """
    widget.hide()
    return True # dont kill the window

  def main_close(self, widget, event=None):
    """ Wrapper for quitting """
    if gtk.main_level() > 1:
      gtk.main_quit()
      gobject.idle_add(self.main_close, 100)
    else:
      # do normal stuff I do when the app quits...
      try:
        gtk.main_quit()
      except RuntimeError, msg:
        verbose('gtk.main_quit() RuntimeError: %s' % msg)
        pass
    return False


  ## FILE MENU ##
  def on_show_features1_activate(self, widget):
    """ File > Show Features """
    self.refreshFeatures()
    self.ui.features.show()

  def on_remove_user_script1_activate(self, widget):
    """ File > Remove User Script """
    result = rat.hig.ok_cancel('Are you sure?', 'This will remove all previous versions currently installed.', title='Are you sure?')
    if result != gtk.RESPONSE_OK:
      return False
    for i in [\
    '%s/.gnome2/nautilus-scripts/audio-convert-mod/' % USERHOME,\
    '%s/.audio-convert-mod/acm-script.sh' % USERHOME,
    '%s/.audio-convert-mod/remove-converted.sh' % USERHOME,
    '%s/.kde/share/apps/konqueror/servicemenus/audio-convert-mod.desktop' % USERHOME,\
    '%s/.kde/share/apps/konqueror/servicemenus/audio-convert-mod-nofolder.desktop' % USERHOME,\
    '%s/.kde/share/apps/konqueror/servicemenus/audio-convert-mod-kde.desktop' % USERHOME\
             ]:
      if os.path.exists(i) and os.path.isfile(i):
        try:
          verbose(_('Removing file from older release `%s\'.' % i))
          os.remove(i)
        except Exception, error:
          verbose(error)
          pass
      if os.path.exists(i) and os.path.isdir(i):
        try:
          verbose(_('Removing directory from older release `%s\'.' % i))
          shutil.rmtree(i)
        except Exception, error:
          verbose(error)
          pass
    return True

  def on_install_user_script1_activate(self, widget):
    """ File > Install User Script """
    if self.on_remove_user_script1_activate(None) == False:
      return
    # KDE
    if os.path.exists('%s/.kde/share/apps/konqueror/servicemenus/' % USERHOME) and os.path.exists('/usr/share/audio-convert-mod/audio-convert-mod-kde.desktop'):
      shutil.copy('/usr/share/audio-convert-mod/audio-convert-mod-kde.desktop', '%s/.kde/share/apps/konqueror/servicemenus/' % USERHOME)
    else:
      verbose(_('Skipping KDE configuration: KDE servicemenu file or directory is missing'))
    # Gnome
    if os.path.exists('%s/.gnome2/nautilus-scripts/' % USERHOME) and os.path.exists('/usr/bin/audio-convert-mod'):
      if not os.path.exists('%s/.gnome2/nautilus-scripts/audio-convert-mod' % USERHOME):
        os.mkdir('%s/.gnome2/nautilus-scripts/audio-convert-mod' % USERHOME)
      os.symlink('/usr/bin/audio-convert-mod', '%s/.gnome2/nautilus-scripts/audio-convert-mod/%s' % (USERHOME, _('Convert Files')))
    else:
      verbose(_('Skipping Gnome configuration: Gnome servicemenu file or directory is missing'))

  def on_quit1_activate(self, widget):
    """ File > Quit """
    self.main_close(widget)

  ## EDIT MENU ##
  def on_preferences1_activate(self, widget):
    """ Open preferences """
    prefs = _readPrefs()
    if int(prefs.get('Preferences', 'ShowTrayIcon')):
      self.ui.prefsShowTrayIconCheck.set_active(True)
    else:
      self.ui.prefsShowTrayIconCheck.set_active(False)

    if int(prefs.get('Preferences', 'BlinkTrayIcon')):
      self.ui.prefsBlinkTrayCheck.set_active(True)
    else:
      self.ui.prefsBlinkTrayCheck.set_active(False)

    if int(prefs.get('Preferences', 'PauseOnErrors')):
      self.ui.prefsPauseOnErrorsCheck.set_active(True)
    else:
      self.ui.prefsPauseOnErrorsCheck.set_active(False)
    self.ui.prefs.show()
    

  ## HELP MENU ##

  def on_about1_activate(self, widget):
    """ Help > About """
    self.ui.about.show()

  ###########
  ## PREFS ##
  ###########

  def on_prefsShowTrayIconCheck_clicked(self, widget):
    prefs = _readPrefs()
    active = self.ui.prefsShowTrayIconCheck.get_active()
    if active:
      prefs.set('Preferences', 'ShowTrayIcon', 1)
    else:
      prefs.set('Preferences', 'ShowTrayIcon', 0)
    _writePrefs(prefs)
    self.trayicon.set_visible(active)
    self.ui.show_tray_icon1.set_active(active)
    if not self.ui.main.get_property('visible'):
      self.ui.main.show()

  def on_prefsBlinkTrayCheck_clicked(self, widget):
    prefs = _readPrefs()
    active = self.ui.prefsBlinkTrayCheck.get_active()
    if active:
      prefs.set('Preferences', 'BlinkTrayIcon', 1)
    else:
      prefs.set('Preferences', 'BlinkTrayIcon', 0)
      self.trayicon.set_blinking(False)
    _writePrefs(prefs)

  def on_prefsPauseOnErrorsCheck_clicked(self, widget):
    prefs = _readPrefs()
    active = self.ui.prefsShowTrayIconCheck.get_active()
    if active:
      prefs.set('Preferences', 'PauseOnErrors', 1)
    else:
      prefs.set('Preferences', 'PauseOnErrors', 0)
    _writePrefs(prefs)

  def on_prefsCloseButton_clicked(self, widget):
    """ close prefs """
    self.ui.prefs.hide()

  ###########
  ## ABOUT ##
  ###########

  def on_aboutWebsiteButton_clicked(self, widget):
    webbrowser.open_new('http://www.diffingo.com/opensource')

  def on_aboutCloseButton_clicked(self, widget):
    self.ui.about.hide()

  ##############
  ## FEATURES ##
  ##############

  def on_featuresCloseButton_clicked(self, widget):
    """ Close features window """
    self.ui.features.hide()

  def on_featuresRefreshButton_clicked(self, widget):
    """ Refresh features """
    self.checkFormats()
    self.refreshFeatures()

  ##################
  ## ERRORDETAILS ##
  ##################

  def on_errorDetailsCloseButton_clicked(self, widget):
    """ Close errorDetails """
    self.ui.errorDetails.hide()

  ##########
  ## MAIN ##
  ##########

  def on_mainBackButton_clicked(self, widget):
    """ Advance to previous page """
    currPage = self.ui.mainControlNotebook.get_current_page()
    if self.trayicon.get_blinking():
      self.trayicon.set_blinking(False)
    self.ui.mainControlNotebook.set_current_page(currPage - 1)
    if currPage == 1:
      # if we hit back on second page
      self.ui.mainBackButton.set_sensitive(False)
      self.ui.main2SaveDefaultsButton.hide()
    elif currPage == 2:
      # if we hit back on third page
      self.ui.main2SaveDefaultsButton.show()
    self.ui.mainNextButton.set_sensitive(True)

  def on_mainNextButton_clicked(self, widget):
    """ Advance to next page """
    currPage = self.ui.mainControlNotebook.get_current_page()
    if self.trayicon.get_blinking():
      self.trayicon.set_blinking(False)
    if currPage == 0:
      # on the first page with no files in the treeview
      model = self.ui.main1FilesTreeview.get_model()
      array = audio_convert_mod.liststoreIntoArray(model)
      if array == []:
        rat.hig.error('Invalid Input', 'Please select at least one file to convert.', title=_('Invalid input') )
        return
      else: # retrieve preferences
        prefs = _readPrefs()
        formatName = prefs.get('Defaults', 'Format')
        quality = prefs.get('Defaults', 'Quality')
        extension = prefs.get('Defaults', 'Extension')
        whenExists = prefs.get('Defaults', 'FileExists')
        metadata = prefs.get('Defaults', 'Metadata')
        successfulConversion = prefs.get('Defaults', 'successfulConversion')
        outputFolder = prefs.get('Defaults', 'outputFolder')
        active = 0
        for format in formats.FORMATS:
          if format.get()[1] == True: # if it's available
            if format.__class__.__name__.upper() == formatName.upper(): # and the name matches
              self.ui.main2FormatCombobox.set_active(int(active)) # make it active
              active = 0 # restart for quality
              for qualitySet in format.get()[2]: # for i in format.__qualities
                if qualitySet[0] == quality:
                  self.ui.main2QualityCombobox.set_active(int(active))
                else:
                  active += 1
              break # exit the loop now that we've found our result
            else:
              active += 1 # if not found, move onto next active item
        self.ui.main2ExtensionCombobox.set_active(int(extension))
        self.ui.__getattr__('main2ExistsRadio%s' % metadata).set_active(True)
        self.ui.__getattr__('main2MetatagRadio%s' % whenExists).set_active(True)
        self.ui.__getattr__('main2UponSuccessRadio%s' % successfulConversion).set_active(True)
        if outputFolder.lower() == 'Off'.lower():
          self.ui.main2OutputFolderRadio1.set_active(True)
        else:
          self.ui.main2OutputFolderFilechooser.set_current_folder(outputFolder)
      # either way, we need to show button...
      self.ui.mainBackButton.set_sensitive(True)
      self.ui.main2SaveDefaultsButton.show()
    elif currPage == 2:
      # if we hit next on conversion page
      self.ui.mainNextButton.set_sensitive(False)
      self.ui.mainBackButton.set_sensitive(False)
    self.ui.mainControlNotebook.set_current_page(currPage + 1)
    if currPage == 1: # this comes on it's own because it needs to happen _after_ page change
      # if we hit next on second page
      self.ui.main2SaveDefaultsButton.hide()
      self.ui.mainNextButton.set_sensitive(False)
      self.ui.mainBackButton.set_sensitive(False)
      try:
        self.startConversion()
      except:
        import traceback
        etb = traceback.extract_tb(sys.exc_info()[2])
        traceback = _('Traceback:\n')
        for tub in etb:
          f, l, m, c = tub # file, lineno, function, codeline
          traceback += _('File: %(a)s, line %(b)s, in %(c)s\n') % {'a': f, 'b': l, 'c': m}
          traceback += ' %s \n' % c
        traceback += '%s: %s\n' % (sys.exc_info()[0], sys.exc_info()[1]) #etype, evalue
        self.ui.errorDetails.show()
        audio_convert_mod.set_text_markup(self.ui.errorDetailsTitleLabel, _('<span size="large" weight="bold">Conversion Error</span>'))
        self.ui.errorDetails.set_title(_('Conversion Error'))
        self.ui.errorDetailsDescriptionLabel.set_text(_('An unexpected error occurred while converting files.\n\
The converted files may left in an incomplete state.'))
        self.ui.errorDetailsTextview.get_buffer().set_text(traceback)
        pass

  def on_main2SaveDefaultsButton_clicked(self, widget):
    """ Save preferences """
    prefs = _readPrefs()
    prefs.set('Defaults', 'Format', self.ui.main2FormatCombobox.get_active_text())
    model = self.ui.main2QualityCombobox.get_model()
    iter = self.ui.main2QualityCombobox.get_active_iter()
    prefs.set('Defaults', 'Quality', model.get_value(iter, 0))
    prefs.set('Defaults', 'Extension', self.ui.main2ExtensionCombobox.get_active())
    active = None
    if self.ui.main2ExistsRadio1.get_active():
      active = 1
    elif self.ui.main2ExistsRadio2.get_active():
      active = 2
    elif self.ui.main2ExistsRadio3.get_active():
      active = 3
    prefs.set('Defaults', 'FileExists', active)
    active = None
    if self.ui.main2MetatagRadio1.get_active():
      active = 1
    elif self.ui.main2MetatagRadio2.get_active():
      active = 2
    prefs.set('Defaults', 'Metadata', active)
    active = None
    if self.ui.main2UponSuccessRadio1.get_active():
      active = 1
    elif self.ui.main2UponSuccessRadio2.get_active():
      active = 2
    elif self.ui.main2UponSuccessRadio3.get_active():
      active = 3
    prefs.set('Defaults', 'successfulConversion', active)
    if self.ui.main2OutputFolderRadio1.get_active():
      prefs.set('Defaults', 'outputFolder', 'Off')
    elif self.ui.main2OutputFolderRadio2.get_active():
      prefs.set('Defaults', 'outputFolder', self.ui.main2OutputFolderFilechooser.get_current_folder())
    _writePrefs(prefs)

  def on_main1AddButton_clicked(self, widget):
    """ Add file(s) """
    pathBrowser = widgets.PathBrowser(self.ui.chooser)
    pathBrowser.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
    response = pathBrowser.run()
    paths = pathBrowser.get_filenames()
    pathBrowser.destroy()
    if response != gtk.RESPONSE_OK:
      return
    errors = widgets.addToSelectFilesFilesTreeview(self, paths)
    if errors != []:
      self.ui.errorDetails.show()
      audio_convert_mod.set_text_markup(self.ui.errorDetailsTitleLabel, _('<span size="large" weight="bold">Error adding files</span>'))
      self.ui.errorDetails.set_title(_('Error adding files'))
      self.ui.errorDetailsDescriptionLabel.set_text(_('Not all files could be added.'))
      self.ui.errorDetailsTextview.get_buffer().set_text('\n'.join(errors))

  def on_main1AddDirectoryButton_clicked(self, widget):
    """ Add files by selecting a folder """
    pathBrowser = widgets.PathBrowser(self.ui.chooser)
    pathBrowser.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
    response = pathBrowser.run()
    paths = pathBrowser.get_filenames()
    paths.sort()
    pathBrowser.destroy()
    if response != gtk.RESPONSE_OK:
      return
    fileList = []
    for folder in paths:
      files = os.listdir(folder)
      files.sort()
      for filename in files:
        if not os.path.isdir('%s/%s' % (folder, filename)):
          fileList.append('%s/%s' % (folder, filename) )
    errors = widgets.addToSelectFilesFilesTreeview(self, fileList)
    if errors != []:
      self.ui.errorDetails.show()
      audio_convert_mod.set_text_markup(self.ui.errorDetailsTitleLabel, _('<span size="large" weight="bold">Error adding files</span>'))
      self.ui.errorDetails.set_title(_('Error adding files'))
      self.ui.errorDetailsDescriptionLabel.set_text(_('Not all files could be added.'))
      self.ui.errorDetailsTextview.get_buffer().set_text('\n'.join(errors))

  def on_main1RemoveButton_clicked(self, widget):
    """ Remove all selected files """
    model, paths = self.ui.main1FilesTreeview.get_selection().get_selected_rows()
    for i in paths:
      model, paths = self.ui.main1FilesTreeview.get_selection().get_selected_rows()
      model.remove(model.get_iter(paths[0]))

  def on_main2FormatRefreshButton_clicked(self, widget):
    """ Refresh the list of valid formats """
    self.checkFormats()

  def on_main2FormatCombobox_changed(self, widget):
    """ The format changed, update qualities and extensions combobox """
    formatModel = self.ui.main2FormatCombobox.get_model()
    formatActive = self.ui.main2FormatCombobox.get_active()
    if formatActive == -1:
      return
    formatIter = formatModel.get_iter(formatActive)
    formatValue = formatModel.get_value(formatIter, 0).split(' ')[0]
    if formatValue == 'MP3' or formatValue == 'OGG' or formatValue == 'FLAC' or formatValue == 'AAC': # supports tags
      self.ui.main2MetatagRadio1.set_sensitive(True)
      self.ui.main2MetatagRadio2.set_sensitive(True)
    else:
      self.ui.main2MetatagRadio1.set_sensitive(False)
      self.ui.main2MetatagRadio2.set_sensitive(False)

    qualityModel = self.ui.main2QualityCombobox.get_model()
    qualityModel.clear()

    extensionModel = self.ui.main2ExtensionCombobox.get_model()
    extensionModel.clear()

    for format in formats.FORMATS:
      if formatValue.lower() == format.__class__.__name__:
        for row in format.get()[2]:
          qualityModel.append(row)
        for i in format.extensions:
          extensionModel.append(['.%s' % i])
    self.ui.main2QualityCombobox.set_active(0)
    self.ui.main2ExtensionCombobox.set_active(0)

  def on_main2OutputFolderFilechooser_current_folder_changed(self, widget):
    """ ... """
    self.ui.main2OutputFolderRadio2.set_active(True)

  def startConversion(self):
    """ Runs the conversion """

    def callback(model, path, iter, user_data):
      """ Append value to user_data """
      files = user_data[0]
      directory = model.get_value(iter, 1)
      filename = model.get_value(iter, 2)
      files.append('%s/%s' % (directory, filename))

    def updateProgress(user_data):
      """ Updates current file progress """
      self = user_data
      if self.fraction == -1:
        # this means we're pulsing, so don't interfere.
        self.ui.main3FileProgress.set_text('Please wait...')
        return True
      if self.fraction == 100 or self.fraction == "":
        self.ui.main3FileProgress.set_fraction(float(1))
        self.ui.main3FileProgress.set_text('100%')
        verbose('No longer updating the progressbar')
        return False
      self.ui.main3FileProgress.set_fraction(float(self.fraction))
      self.ui.main3FileProgress.set_text('%i%%' % int(float(self.fraction)*100))
      #audio_convert_mod.doGtkEvents()
      return True

    def escape(name):
      """ escape special characters """
      return '&amp;'.join(name.split('&'))

    def escapeSingleQuote(name):
      """ escape single quotes """
      return '\'\\\'\''.join(name.split('\''))

    def badExitValue(sub):
      """ Popup notification for bad exit value """
      rat.hig.warning(_('Exit value'), _('The process used for decoding files returned a non-zero value `%i`.\nThis usually means there was an error while running the conversion, or the process was stopped unexpectedly.' % sub.poll()), title=_('Exit value'))
      verbose(sub.stderr.readlines())
      verbose(sub.stdout.readlines())
      verbose(command)

    def refreshCurrentFile(self, i, currFile, total):
      """ Refreshes labels for the current file """
      self.ui.main3CurrentFileLabel.set_label(i.split('/')[-1])
      self.ui.main3ConvertProgress.set_text( _('%(a)i/%(b)i file(s) completed' % {'a': currFile, 'b': total} ) )
      self.ui.main3ConvertProgress.set_fraction(float(currFile)/float(total))
      #audio_convert_mod.doGtkEvents()

    self.processId = None
    self.ui.main3SkipFileButton.set_sensitive(True)
    self.ui.main3CancelButton.set_sensitive(True)
    # get output name and object
    outputFormat = self.ui.main2FormatCombobox.get_active_text()
    for format in formats.FORMATS:
      outputFormatObject = None
      if format.__class__.__name__ == outputFormat.lower():
        outputFormatObject = format
        break
    if outputFormatObject == None:
      rat.hig.error(_('Lookup Error'), _('Critical error: Could not look up the encode object for the file type `%s\'.' % outputFormat.lower()), title=_('Lookup Error'))
      self.main_close(None)

    self.setStatus(_('Idle'))

    # decode only?
    self.decodeonly = False
    if outputFormat == 'WAV':
      self.decodeonly = True

    # quality for label
    iter = self.ui.main2QualityCombobox.get_active_iter()
    quality = self.ui.main2QualityCombobox.get_model().get_value(iter, 1)
    self.ui.main3ConvertToLabel.set_text('%(a)s @ %(b)s' % {'a': outputFormat, 'b': quality})
    # numberic quality only
    quality = int(self.ui.main2QualityCombobox.get_model().get_value(iter, 0))
    # setup
    files = []
    model = self.ui.main1FilesTreeview.get_model()
    model.foreach(callback, [files])
    self.fraction = 0
    # update file progress every 100 miliseconds
    gobject.timeout_add(100, updateProgress, self)
    total = len(files)
    currFile = 0
    progressBar = widgets.ProgressBar(self.ui.main3FileProgress)
    for i in files:
      if self.processId == -1:
        # don't continue onto next file if we hit cancel
        return
      self.setStatus(_('Initializing'))
      # get input format and object
      inputFormat = formats.getFileType(i).__class__.__name__.upper()
      inputFormatObject = formats.getFileType(i)

      # encode only?
      self.encodeonly = False
      if inputFormat == 'WAV':
        self.encodeonly = True
      # update file labels
      refreshCurrentFile(self, i, currFile, total)

      #decode
      if outputFormat == 'MP3' or outputFormat == 'FLAC': # only two of four because other two set tags during encode
        verbose('get tags')
        self.setStatus(_('Retrieving tags/metadata'))
        if self.ui.main2MetatagRadio1.get_active() and inputFormat != 'WAV':
          tags = inputFormatObject.getTags(i)
        elif self.ui.main2MetatagRadio2.get_active():
          self.ui.tags.show()
          title = self.ui.tagsTitleEntry.set_text('')
          artist = self.ui.tagsArtistEntry.set_text('')
          album = self.ui.tagsAlbumEntry.set_text('')
          year = self.ui.tagsYearEntry.set_text('')
          track = self.ui.tagsTrackEntry.set_text('')
          genre = self.ui.tagsGenreEntry.set_text('')
          bounds = self.ui.tagsCommentsTextview.get_buffer().set_text('')
          response = self.ui.tags.run()
          self.ui.tags.hide()
          title = self.ui.tagsTitleEntry.get_text()
          artist = self.ui.tagsArtistEntry.get_text()
          album = self.ui.tagsAlbumEntry.get_text()
          year = self.ui.tagsYearEntry.get_text()
          track = self.ui.tagsTrackEntry.get_text()
          genre = self.ui.tagsGenreEntry.get_text()
          bounds = self.ui.tagsCommentsTextview.get_buffer().get_bounds()
          comments = self.ui.tagsCommentsTextview.get_buffer().get_text(bounds[0], bounds[1])
          tags = [title, artist, album, year, track, genre, comments]
        else:
          tags = None
      else:
        tags = None
      if not self.encodeonly:
        verbose('decode')
        self.setStatus(_('Decoding'))
        sub, command = inputFormatObject.decode(escapeSingleQuote(i))
        verbose(command)
        self.processId = sub.pid
        if inputFormat == 'AC3': # a52dec doesn't support progress
          self.fraction = -1
          progressBar.startPulse()
          while sub.poll() == None:  
            audio_convert_mod.doGtkEvents()
          progressBar.stopPulse()
        else:
          while sub.poll() == None:
            try:
              if self.decodeonly:
                self.fraction = float(''.join(sub.stdout.readline().split('\n')[:-1]))
              else:
                self.fraction = float(''.join(sub.stdout.readline().split('\n')[:-1]))/2
            except ValueError:
              self.fraction = '.5'
            audio_convert_mod.doGtkEvents()
        if sub.poll() != 0 and sub.poll() != -9:
          badExitValue(sub)

        time.sleep(0.1)
      if self.processId == -1:
        # don't continue onto next file if we hit cancel
        return
      if self.processId == -2:
        # skip current file
        currFile += 1
        continue

      #options
      verbose('get options')
      self.setStatus(_('Retrieving options'))
      extension = self.ui.main2ExtensionCombobox.get_active_text().strip('.')
      if self.ui.main2OutputFolderRadio1.get_active():
        newname = formats.getNewExt(extension.lower(), escapeSingleQuote(i))
      elif self.ui.main2OutputFolderRadio2.get_active():
        outputFolder = self.ui.main2OutputFolderFilechooser.get_current_folder()
        newname = os.path.join( outputFolder, os.path.split( formats.getNewExt(extension.lower(), escapeSingleQuote(i)) )[1])
      else: # just incase
        newname = formats.getNewExt(extension.lower(), escapeSingleQuote(i))

      while os.path.exists(newname):
        if self.ui.main2ExistsRadio1.get_active(): # append .converted
          newname += '.converted'
          if outputFormat == 'MPC' and not newname.endswith('.mpc'):
            # a stupid hack because mpc can't encode to something that doesn't end in .mpc
            newname += '.mpc'
        elif self.ui.main2ExistsRadio3.get_active() and os.path.exists(newname) and outputFormat != 'WAV': #overwrite
          try:
            verbose('removing existing file %s' % newname)
            os.remove(newname)
          except Exception, error:
            verbose(error)
        elif self.ui.main2ExistsRadio4.get_active(): #move to trash
          try:
            directory, filename = '/'.join(i.split('/')[:-1]), i.split('/')[-1]
            verbose('moving %s to trash'  % newname)
            shutil.move( newname, '%s/.Trash/%s' % (USERHOME, filename) )
          except Exception, error:
            rat.hig.warning(_('File Error'), _('An error occurred while attempting to move the file `%s\' to the trash. Please make sure ~/.Trash exists and is a directory.\n\nBecause of this error, \'.converted\' will be appended to the new files to prevent the original from being overwritten.' % escape(filename)), title=_('File Error'))
            verbose(error)
            newname += '.converted'
        else: # we should NEVER be here, but just incase.
          newname += '.converted'
      if not self.decodeonly:
        #encode
        verbose('encode')
        verbose('encoding %s to %s @ %s' % (formats.getNewExt('wav', i), newname, quality))
        self.setStatus(_('Encoding'))
        if outputFormat == 'AAC' or outputFormat == 'OGG':
          sub, command = outputFormatObject.encode(escapeSingleQuote(formats.getNewExt('wav', i)), newname, quality, tags)
        else:
          sub, command = outputFormatObject.encode(escapeSingleQuote(formats.getNewExt('wav', i)), newname, quality)
        verbose(command)
        self.processId = sub.pid
        if outputFormat == 'AC3': # ffmpeg doesn't support progress
          self.fraction = -1
          progressBar.startPulse()
          while sub.poll() == None:
            audio_convert_mod.doGtkEvents()
          progressBar.stopPulse()
        else:
          while sub.poll() == None:
            try:
              if self.encodeonly:
                self.fraction = float(''.join(sub.stdout.readline().split('\n')[:-1]))
              else:
                self.fraction = float(''.join(sub.stdout.readline().split('\n')[:-1]))/2 + .5
            except ValueError:
              self.fraction = 1
            audio_convert_mod.doGtkEvents()
        if sub.poll() != 0 and sub.poll() != -9:
          rat.hig.warning(_('Exit value'), _('The process used for encoding files returned the non-zero value `%i`.\nThis usually means there was an error while running the conversion, or the process was stopped unexpectedly.' % sub.poll()), title=_('Exit value'))
          verbose(sub.stderr.readlines())
          verbose(sub.stdout.readlines())
          verbose(command)
      #tags
      verbose(tags)
      self.setStatus(_('Setting tags/metadata'))
      if tags:
        if outputFormat == 'MP3' or  outputFormat == 'OGG' or outputFormat == 'FLAC' or outputFormat == 'AAC':
          outputFormatObject.setTags(newname, tags)

      # setup for next file
      self.setStatus(_('Cleanup'))
      if outputFormat != 'WAV' and inputFormat != 'WAV': # if the source or out is wave, don't delete!
        verbose('removing wav file `%s\'' % formats.getNewExt('wav', i))
        try:
          os.remove(formats.getNewExt('wav', i))
        except OSError:
          verbose('File `%s\' not found! Ignoring.' % formats.getNewExt('wav', i))
      if os.path.exists(newname):
        if outputFormat == 'MPC' and newname.endswith('.mpc.converted.mpc'):
          # a stupid hack because mpc can't encode to something that doesn't end in .mpc
          try:
            os.rename(newname, '.converted'.join(newname.split('.converted.mpc')))
          except OSError:
            verbose('Error while moving `%s\' to `%s\'... Ignoring.' % (newname, '.converted'.join(newname.split('.converted.mpc'))))
      if sub.poll() == 0:
        if os.path.exists(i):
          if self.ui.main2UponSuccessRadio2.get_active():
            verbose('moving original file `%s\' to trash' % i)
            try:
              shutil.move(i, '%s/.Trash/%s' % (USERHOME, os.path.split(i)[1]))
            except OSError:
              verbose('File `%s\' not found! Ignoring.' % i)
          elif self.ui.main2UponSuccessRadio3.get_active():
            verbose('removing original file `%s\'' % i)
            try:
              os.remove(i)
            except OSError:
              verbose('File `%s\' not found! Ignoring.' % i)
      else:
        verbose('Bad exit status %s on conversion `%s\', skipping (re)move options if any' % (sub.poll(), i))
      currFile += 1
      self.ui.main3FileProgress.set_fraction(float(1))
      self.ui.main3FileProgress.set_text('100%')
      self.setStatus(_('Idle'), blink=True)
      audio_convert_mod.doGtkEvents()
      time.sleep(0.1)
    self.fraction = 100
    self.ui.main3ConvertProgress.set_text( _('%(a)i/%(b)i file(s) completed' % {'a': currFile, 'b': total} ) )
    self.ui.main3ConvertProgress.set_fraction(float(currFile)/float(total))
    self.ui.main3SkipFileButton.set_sensitive(False)
    self.ui.main3CancelButton.set_sensitive(False)
    self.ui.mainBackButton.set_sensitive(True)
    self.ui.mainNextButton.set_sensitive(True)

  def on_main3CancelButton_clicked(self, widget):
    """ Cancel All """
    self.on_main3SkipFileButton_clicked(None)
    self.processId = -1
    self.ui.main3ConvertProgress.set_text(_('Cancelled.'))
    self.ui.main3ConvertProgress.set_fraction(float(1))
    self.ui.mainBackButton.set_sensitive(True)
    self.ui.mainNextButton.set_sensitive(True)


  def on_main3SkipFileButton_clicked(self, widget):
    """ Skip current file """
    self.ui.main3FileProgress.set_text(_('Cancelled.'))
    self.ui.main3FileProgress.set_fraction(float(1))
    if self.processId >= 2 and self.processId != None:
      try:
        os.kill(self.processId, 9)
      except:
        verbose('Could not kill process')
    self.processId = -2

  def on_main4NewButton_clicked(self, widget):
    """ Start over """
    self.checkFormats()
    self.refreshFeatures()
    self._setDefaults()
    self.ui.mainBackButton.set_sensitive(True)
    self.ui.mainNextButton.set_sensitive(True)

  def on_main4QuitButton_clicked(self, widget):
    """ Quit """
    self.main_close(None)


